<?php declare(strict_types=1);
/**
 * NOTICE OF LICENSE
 *
 * @copyright  Copyright (c) 29.10.2021 brainstation GbR
 * @author     Mike Becker<mike@brainstation.de>
 */
namespace BstInstaShoppingModify6\Storefront\Subscriber;

use BstInstaShopping6\Event\InstaShoppingCsvHeaderBeforeEvent;
use BstInstaShopping6\Event\InstaShoppingCsvLineItemBeforeEvent;
use BstInstaShopping6\Event\InstaShoppingProductsCriteriaEvent;
use BstInstaShopping6\Event\InstaShoppingProductsLineItemBeforeEvent;
use BstInstaShopping6\Event\InstaShoppingProductsLineItemsEvent;
use BstInstaShopping6\Event\InstaShoppingProductsProcessingBeforeEvent;
use BstInstaShopping6\Settings\Struct\SettingsStruct;
use BstInstaShopping6\Struct\LineItemStruct;
use Psr\Log\LoggerInterface;
use Shopware\Core\Content\Product\ProductEntity;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class Export implements EventSubscriberInterface
{
    /** @var LoggerInterface */
    private $logger;

    /**
     * Export constructor.
     *
     * @param LoggerInterface $logger
     */
    public function __construct(
        LoggerInterface $logger
    )
    {
        $this->logger = $logger;
    }

    /**
     * @return array
     */
    public static function getSubscribedEvents(): array
    {
        return [
            InstaShoppingProductsCriteriaEvent::class => 'onProductsCriteria',
            InstaShoppingProductsLineItemBeforeEvent::class => 'onLineItemBefore',
            InstaShoppingProductsLineItemsEvent::class => 'onLineItems',
            InstaShoppingProductsProcessingBeforeEvent::class => 'onProductsProcessingBefore',
            InstaShoppingCsvHeaderBeforeEvent::class => 'onCsvHeaderBefore',
            InstaShoppingCsvLineItemBeforeEvent::class => 'onCsvLineItemBefore',
        ];
    }

    /**
     * @param InstaShoppingProductsCriteriaEvent $event
     */
    public function onProductsCriteria(InstaShoppingProductsCriteriaEvent $event): void
    {
        $salesChannelContext = $event->getSalesChannelContext();
        $criteria = $event->getCriteria();
    }

    /**
     * @param InstaShoppingProductsLineItemBeforeEvent $event
     */
    public function onLineItemBefore(InstaShoppingProductsLineItemBeforeEvent $event): void
    {
        /** @var ProductEntity $product */
        $product = $event->getProduct();

        /** @var LineItemStruct $lineItem */
        $lineItem = $event->getLineItem();

        /** @var array $propertyData */
        $propertyData = $event->getPropertyData();

        /** @var SettingsStruct $settings */
        $settings = $event->getSettings();

        // Example: Check if product is on sale. In this case export SALE Category for product_type
//        if ($product->getPrice()->first()->getListPrice()) {
//            $lineItem->setCategoryPath('SALE');
//        }

        $event->setLineItem($lineItem);
    }

    /**
     * @param InstaShoppingProductsCriteriaEvent $event
     */
    public function onLineItems(InstaShoppingProductsLineItemsEvent $event): void
    {
        $lineItems = $event->getLineItems();

        // Example: append _mod for product names
//        $lineItemsMod = [];
//        foreach ($lineItems as $item) {
//            $item[2] = $item[2] . '_mod';
//            $lineItemsMod[] = $item;
//        }
//        $lineItems = $lineItemsMod;

        $event->setLineItems($lineItems);
    }

    /**
     * @param InstaShoppingProductsProcessingBeforeEvent $event
     */
    public function onProductsProcessingBefore(InstaShoppingProductsProcessingBeforeEvent $event): void
    {
        $iterator = $event->getIterator();
        $productResult = $event->getProductResult();
        $salesChannelDomain = $event->getSalesChannelDomain();

        // modify result before processing
    }

    /**
     * @param InstaShoppingCsvHeaderBeforeEvent $event
     */
    public function onCsvHeaderBefore(InstaShoppingCsvHeaderBeforeEvent $event): void
    {
        $header = $event->getHeader();

        // modify header before processing
        // Example: append new header column
//        $header[] = 'Custom Column';
        $event->setHeader($header);

    }

    /**
     * @param InstaShoppingCsvLineItemBeforeEvent $event
     */
    public function onCsvLineItemBefore(InstaShoppingCsvLineItemBeforeEvent $event): void
    {
        $lineItemFields = $event->getFields();

        // Example: append new lineItem column
//        $lineItemFields[] = 'Field value';
        $event->setFields($lineItemFields);
    }
}
